
package sub_arctic.constraints;


/** 
 * An interface for objects that receive (consume) values within the 
 * "external value" system.  This system bridges the lightweight constraint 
 * system to external entities -- either to non-local dependents, to 
 * heavyweight constraints, or to "application" objects.  Value_consumer 
 * objects may maintain several parts (each with a unique integer designator).
 * Each of these parts may be dependent on (or interested in) one or more
 * parts of various value_provider objects.  Value_consumer objects register
 * themselves with value_value_provider objects using the attach_dependent()
 * method.  Subsequently, they relieve value_ood() messages whenever the
 * value in question changes (or might have changed).  At an appropriate 
 * point, the consumer object may then use the get_value() method to retrieve
 * an updated value.
 *
 * @author Scott Hudson
 */
public interface value_consumer {
  /** 
   * Indicate that a value (a part of a value_provider object) has, or might 
   * have, changed value.  This object can use the get_value() method to
   * retrieve the updated value.  However, this method should not immediately
   * request that value.  Instead it must wait until all marking is complete.
   * Inside the constraint system this is handled by waiting for the value to
   * be demanded.  In other circumstances this may require a request for 
   * evaluation to be queued elsewhere.<p>
   * 
   * @param int            for_part_here the part number within this object 
   *                                     being informed that it may be out of
   *                                     date.
   * @param value_provider prov_obj      the object providing the value that 
   *                                     may have changed.
   * @param int            prov_part     the part number within that object 
   *                                     which have changed value.
   */
   public void value_ood(
     int            for_part_here, 
     value_provider prov_obj, 
     int            prov_part);

   //had:
   //* @exception bad_value if the object is null or one of the parts is not a
   //*                      valid part number for the object it is associated 
   //*                      with.
   //* @exception general
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
