
package sub_arctic.input;

import sub_arctic.lib.interactor;

/** 
 * Focus agent that implements inout-drag.  inout_drag is a drag which only
 * tracks entry and exit from the bounds of an object.  The inout_draggable 
 * protocol provides for inout_drag_start(), inout_drag_enter(), 
 * inout_drag_exit(), and inout_drag_end() methods. <p> 
 *
 * @see sub_arctic.input.inout_draggable
 * @author  Scott Hudson
 */

public class inout_drag_focus_agent extends single_focus_agent {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /* Simple constructor */
  public inout_drag_focus_agent()
    {
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Test whether the given object can be added to the focus set (i.e., whether
   * it is inout_draggable).
   *
   * @param focusable candidate_obj
   * @return boolean indicating whether the object can be put in the focus set.
   */
  public boolean allowable_focus(focusable candidate_obj)
    {
      /* must be a inout_draggable */
      return candidate_obj instanceof inout_draggable;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Indicate whether the given event is useful for this agent.  This agent 
   * listens form MOUSE_DRAG, MOUSE_MOVE, and MOUSE_UP events.
   *
   * @param event evt the event we are asking about.
   * @return boolean indicating if the event in useful to this agent.
   */
  public boolean event_is_useful(event evt)
    {
      /* interested in drag, move, and button up */
      return evt.id() == event.MOUSE_DRAG || evt.id() == event.MOUSE_MOVE || 
	     evt.id() == event.MOUSE_UP;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Was the previous event inside the object */
  protected boolean prev_inside = false;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Action taken when an object enters the focus set.  This overrides the
   * super class to call inout_drag_start() instead of focus_set_enter().<p>
   *
   * @param focusable obj       the object being added to the focus set.
   * @param event     evt       the event "causing" the focus.
   * @param Object    user_info the information associated with the object when 
   *                            the focus was established.
   */
  protected void inform_focus_enter(
    focusable obj, 
    event     evt, 
    Object    user_info)
    {
      event copy;

      inout_draggable inout_obj = (inout_draggable)obj;

      /* do an inout_drag_start() rather than focus_set_enter() */

      /* put copy of event into coords of object */
      copy = new event(evt);
      copy.global_to_local(inout_obj);

      /* do inside/outside test and dispatch */
      prev_inside = inout_obj.inside_bounds(copy.local_x(), copy.local_y());
      inout_obj.inout_drag_start(copy, prev_inside, user_info);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Action taken when an object exits the focus set.  This overrides the
   * super class to call inout_drag_end() instead of focus_set_exit().<p>
   *
   * @param focusable obj       the object being removed from the focus set.
   * @param event     evt       the event "causing" the action.
   * @param Object    user_info the information associated with the object when 
   *                            the focus was established.
   */
  protected void inform_focus_exit(
    focusable obj, 
    event     evt, 
    Object    user_info)
    {
      event copy;

      inout_draggable inout_obj = (inout_draggable)obj;

      /* dispatch an inout_drag_end() rather than focus_set_exit() */

      /* put copy of event in local coords and dispatch it */
      copy = new event(evt);
      copy.global_to_local(inout_obj);
      prev_inside = inout_obj.inside_bounds(copy.local_x(), copy.local_y());
      inout_obj.inout_drag_end(copy, prev_inside, user_info);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Attempt to dispatch an event under this agent's protocol.
   *
   * @param event      evt       the event to be dispatched.
   * @param Object     user_info ignored (since we are a focus agent).
   * @param interactor to_obj    ignored (since we are a focus agent).
   * @param int        seq_num   ignored (since we are a focus agent).
   */
  public boolean dispatch_event(
    event      evt,
    Object     user_info,
    interactor to_obj,
    int        seq_num)
    {
      inout_draggable inout_obj;
      boolean        now_inside;
      event          copy;

      /* extract our focus item and if there is none, bail out now */
      inout_obj = (inout_draggable)_focus_set.elementAt(0);
      if (inout_obj == null) return false;

      /* make a fresh copy of the event */
      copy = new event(evt);

      /* If this is end, clear the focus and let inform_focus_exit do work */
      if (copy.id() == event.MOUSE_UP)
	{
	  clear_focus(copy);
	  return true;
	}

      /* put event in local coords of the object and see if we are in or out */
      copy.global_to_local(inout_obj);
      now_inside = inout_obj.inside_bounds(copy.local_x(), copy.local_y());

      /* do we have a change */
      if (now_inside != prev_inside)
	{
	  /* remember the change */
	  prev_inside = now_inside;

	  /* dispatch the change */
	  if (now_inside)
	    return inout_obj.inout_drag_enter(copy,user_info);
	  else
	    return inout_obj.inout_drag_exit(copy,user_info);
	}
      else
	/* we consume the event so it does not sneak into another 
	 * protocol and cause weird effects. 
	 */
	return true;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
