package sub_arctic.input;

import sub_arctic.lib.interactor;
import sub_arctic.lib.manager;

/** 
 * This class implements an agent which is a hybrid positional/focus agent.
 * It basically accepts a positional press over a properly typed object (an
 * interactor which implements move_press_draggable) and hands it off to (i.e.,
 * makes it the focus of) the move_drag agent.
 *
 * @see sub_arctic.input.pressable
 * @see sub_arctic.input.move_draggable
 * @see sub_arctic.input.move_drag_focus_agent
 * @author Scott Hudson
 */
public class move_press_drag_agent extends dispatch_agent {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Simple constructor */
  public move_press_drag_agent()
    {
      /* nothing to do here */
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Declares that we are only interested in press of the mouse button. 
   * Other events are handled by the move_drag agent itself.
   *
   * @param evt the event we are deciding interest in
   * @return whether we are interested in having the event dispatched to us
   */
  public boolean event_is_useful(event evt)
    {
      return evt.id() == event.MOUSE_DOWN;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Attempt to perform event dispatch to the given object.  If the object
   * is move_press_draggable, then we make it the move_drag focus and we 
   * are done.  Otherwise we reject the input.
   *
   * @param evt       the event being dispatched
   * @param user_info an information object to pass with input.  
   *                  This gets repackaged inside an object_holder object 
   *                  and passed on the the move_drag_focus
   * @param to_obj    the object (selected via a pick) we are dispatching to
   * @param seq_num   event sequence number (ignored here)
   * @return whether the event was dispatched successfully
   */
  public boolean dispatch_event(
    event            evt,
    Object           user_info,
    interactor       to_obj,
    int              seq_num)
    {

      /* see if it wants this kind of input */
      if (to_obj instanceof move_press_draggable)
	{
	  /* make the object the move_drag focus, passing an object_holder  
	   * that has the original position user_info in it as the new 
	   * user_info object.
	   */
	  /* put the event into the local coords of the object */
	  evt.global_to_local(to_obj);
	  
	  /* make the object the move_drag_focus */
	  manager.move_drag_focus.set_focus_to(
					       (move_press_draggable)to_obj, evt, new object_holder(user_info));
	  return true;
	}
      else
        return false;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
