package sub_arctic.lib;

import sub_arctic.output.style_manager;
import sub_arctic.output.drawable;
import sub_arctic.constraints.std_function;

import java.util.Vector;
import java.awt.Dimension;

/**
 * This is a helper class for listbox_display. This object does the
 * work of laying out the children in a "columnlike" way. It can
 * reset what children are displayed and what children are selected
 * based on a vector. 
 * 
 * @author Ian Smith
 */
public class listbox_child_display extends base_parent_interactor {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Cached width of this object.
   */
  protected int _cached_width=-1;

  /**
   * Cached height of this object.
   */
  protected int _cached_height=-1;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Store the parent listbox_display of this object.
   */
  protected listbox_display _parent_listbox;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * We keep a cached copy of the set of selected objects so
   * we can make dragging faster. 
   */
  protected Vector _selected_set=new Vector(); /* init so we don't get
						  hosed if they ask for
						  it before they insert data*/

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Retrieve the parent listbox_display of this object.
   * @return listbox the listbox this object is associated with
   */
  public listbox_display parent_listbox() {
    return _parent_listbox;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Create a listbox display. The creator should supply a width 
   * a height. This object has no sizing behavior, it should be
   * clipped by placing it inside another object. We assume that
   * the creator of the listbox will position this object "by
   * hand" if it is necessary for this object to appear at a location
   * other than 0,0 of its parent. 
   *
   * @param int             w  width of this object
   * @param int             h  height of this object
   * @param listbox_display pl the listbox_display this object is embedded in
   */

  public listbox_child_display(int w,int h, listbox_display pl) 
{
    /* setup with respect to parent */
    super(0,0,w,h);
    _parent_listbox=pl;
  }

   //had:
   //* @exception general PROPAGATED

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Put a vector of list_elements into this object. This will
   * replace the current contents of this object.
   *
   * @param Vector contents the new contents of this object (all the objects 
   *                        in this vector must be list_elements).
   */
  public void set_contents(Vector contents) 
{
    int i;
    list_element le;

    /* delete all existing children */
    while (num_children()>0) {
      le=(list_element)child(0);
      le.set_y_constraint(NO_CONSTRAINT);
      remove_child(0);
    }

    /* if the vector is null or empty, just clear things */
    if ((contents==null) || (contents.size()==0)) return;

    /* walk over new children */
    for (i=0; i<contents.size(); ++i) {

      le=(list_element)contents.elementAt(i);

      /* the zeroth child doesn't use constraints */
      if (i==0) {
	/* its y position is just the parent's top */
	le.set_y(0);
      } else {
	/* just derive position from previous sibling */
	le.set_y_constraint(std_function.offset(PREV_SIBLING.Y2(),0));
      }

      /* we always put them against the left */
      le.set_x(0);

      /* put them in our object */
      add_child(le);
    }

    /* we need to recalculate the width */
    calculate_width();
  }

   //had:
   //* @exception general PROPAGATED

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This method sets the set of objects which are selected. All
   * of the objects in this list should be objects which are
   * already in the set of objects in this display. If they are
   * not, they are ignored. However, they objects in this Vector
   * must be list_elements.
   *
   * @param Vector selected_set the objects which should be displayed as 
   *                            selected.
   */
  public void set_selected(Vector new_set) 
{
    list_element obj;
    int i,indx;

    /* clear all current selections */
    for (i=0; i<num_children(); ++i) {
      /* get the object and if its highlighted, make it not so */
      obj=(list_element)child(i);
      if (obj.highlighted()) {
	obj.unhighlight();
      }
    }

    /* is the set null or empty? */
    if ((new_set==null) ||
	(new_set.size()==0)) {
      _selected_set=new Vector();
      return;
    } else {
      /* cache a copy */
      _selected_set=new_set;
    }

    /* loop over all the selected objects */
    for (i=0; i<new_set.size();++i) {
      obj=(list_element)new_set.elementAt(i);

      /* try to find it in the list of children... */
      indx=find_child(obj);

      /* if its not there, just skip it */
      if (indx==-1) continue;

      /* highlight it? */
      obj.highlight();
    }
  }

   //had:
   //* @exception general PROPAGATED

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * We need to inform the parent when our size changes, so we override
   * configure to allow us to detect this.
   */
  public void configure() {

    /* go ahead and do the configure */
    super.configure();

    /* if our width or height changed, we tell the parent */
    if ((w()!=_cached_width)  ||
	(h()!=_cached_height)) {

      /* store the new value */
      _cached_width=w();
      _cached_height=h();

      /* tell the parent */
      parent_listbox().child_size_changed();
    }
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This object wants to have an opaque background which is the color
   * of the text background of the style system. We override draw
   * self local to get this behavior.
   * 
   * @param drawable d the surface to draw on
   */
  public void draw_self_local(drawable d) 
{

    /* fill with the color the labels will use */
    d.setColor(style_manager.default_color_scheme().text_background());
    d.fillRect(0,0,w(),h());

    /* now draw the children */
    draw_children(d); 
  }

   //had:
   //* @exception general PROPAGATED

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Calculate the width by iterating over the children and computing
   * our width to be the largest of the children.
   */
  public void calculate_width() {
    int max=0,i;
    list_element le;
    listbox_helper lh;

    /* if we have a parent, use its width */
    if (parent()!=null) {
      lh=(listbox_helper)parent();
      max=lh.available_width();
    }

    /* loop over our children */
    for (i=0; i<num_children(); ++i) {
      /* convert to list element and see if its bigger than max */
      le=(list_element)child(i);
      if (max<le.preferred_width()) {
	max=le.preferred_width();
      }
    }

    set_w(max);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is a predicate used by other classes to see if this
   * display will fit in an area's width. 
   * @param int width the width we would like to display this object in
   * @return boolean true if this object will fit in that area
   */
  public boolean will_fit_width(int width) {
    int max=0,i;
    list_element le;

    /* loop over our children */
    for (i=0; i<num_children(); ++i) {
      /* convert to list element and see if its bigger than max */
      le=(list_element)child(i);
      if (max<le.preferred_width()) {
	max=le.preferred_width();
      }
    }

    /* did it fit? */
    return (width>=max);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is a predicate used by other classes to see if this
   * display will fit in an area's height;
   * @param int height the height we would like to display this object in
   * @return boolean true if this object will fit in that area
   */
  public boolean will_fit_height(int height) {
    return (height>=h());
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is called by our parent to find the object which will
   * cover a given pixel coordinate. 
   *
   * @param int target the target pixel we are looking for 
   * @return int the index of the object. We return 0 if this object is empty.
   */
  public int index_for_pixel(int target) {
    int i;
    interactor obj=null;

    /* loop backwards, looking for first object that covers this point */
    for (i=num_children()-1; i>=0; --i) {
      obj=child(i);

      /* grab this element and check it */
      if (obj.y()<=target) {
	break;
      }
    }

    /* check for empty */
    if (obj==null) {
      return 0;
    }

    /* normal case */
    return obj.child_index();
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function retrieves the currently selected set of objects
   * as list_elements. This is primarily useful if you wish to address
   * these objects as interactors, rather than as data values.
   *
   * @return Vector a vector of list_elements which are those objects that 
   *                are currently selected. This will return an empty (not 
   *                null) Vector if the selected set is currently empty.
   */
  public Vector selected_set_raw() {
    return _selected_set;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function retrieves the currently selected set of objects
   * as values. list_element objects which support the 
   * "convert_to_object" method will return a more convenient 
   * representation of themselves, and those that done will be returned
   * as list_elements. If nothing is selected, this will return an
   * empty (not null) vector.
   *
   * @return Vector a vector of selected set objects
   */
  public Vector selected_set() {
    Vector v=new Vector();
    int i;
    list_element elem;
    Object obj;

    /* loop over them.. looking for things that are selected */
    for (i=0; i<num_children(); ++i) {

      elem=(list_element)child(i);

      /* is it selected? */
      if (elem.highlighted()) {
	/* get the object we want */
	obj=elem.convert_to_object();
	if (obj==null) {
	  obj=elem;
	}

	/* add this to the vector */
	v.addElement(obj);
      }
    }

    return v;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Retrieve the contents of the listbox transformed by
   * the listbox elements via convert_to_object.
   * If the listbox is empty, this will return an empty Vector.
   * @return Vector the contents of the listbox
   */
  public Vector contents() {
    Vector result=new Vector();
    Object obj;
    int i;
    list_element le;

    /* loop over all children */
    for (i=0; i<num_children(); ++i) {
      /* get the object out */
      le=(list_element)child(i);
      obj=le.convert_to_object();

      /* if that's valid, use that */
      if (obj==null) {
	result.addElement(obj);
      } else {
	/* put it in raw */
	result.addElement(le);
      }
    }

    return result;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Retrieve the contents of the listbox as list_elements.
   * If the listbox is empty, this will return an empty Vector.
   * @return Vector the contents of the listbox
   */
  public Vector contents_raw() {
    Vector result=new Vector();
    Object obj;
    int i;
    list_element le;

    /* loop over all children */
    for (i=0; i<num_children(); ++i) {
      /* add that child to the list */
      result.addElement(child(i));
    }
    return result;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
