
package sub_arctic.lib; 

import sub_arctic.output.drawable;

/** 
 * This class provides a traversal "parameters" object that encapsulates 
 * the context of a draw done from inside a semantic lens.  A semantic
 * lens does a re-rendering of the interactor tree rooted at its parent in
 * a way that changes the display in some significant way.  For example, the
 * nametag_sem_draw redraw pass draws all objects as a small tag displaying
 * the name of the class of the object, along with a bounding box.  These
 * objects are used by the sem_draw_cont predicate, various specialized
 * drawing actions, and the sem_draw_to_child transformer.
 *
 * @see sub_arctic.lib.sem_draw_to_child
 * @see sub_arctic.lib.sem_draw_cont
 * @see sub_arctic.lib.base_interactor#traverse_and_collect
 * @author Scott Hudson
 */
public class sem_draw_context {
  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * A unique identifier that indicates what type of drawing is being done.
   * This value should be allocated by manager.unique_int() to ensure 
   * uniqueness, and stored in a static variable that is accessible to 
   * any interactor that needs to specialize itself on that basis.  
   */
  public int draw_kind;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * The drawable for drawing the result on.  This drawable will have a 
   * translation set up appropriate for the object it is handed to (i.e., 
   * such that 0,0 occurs at the object's top-left), but will not be set
   * to clip to the object (just to the bounds of the semantic lens). 
   */
  public drawable surface;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Constructor.
   * @param ind      drw_kind a value for draw_kind.
   * @param drawable surf     a value for surface.
   */
  public sem_draw_context(int drw_knd, drawable surf)
    {
      draw_kind = drw_knd;
      surface   = surf;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Construct from another object. 
   * @param sem_draw_context other the object we make a copy of. 
   */
  public sem_draw_context(sem_draw_context other)
    {
      this(other.draw_kind, other.surface.copy());
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}

/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
