package sub_arctic.output;

import sub_arctic.lib.*;
import sub_arctic.input.*;
import sub_arctic.constraints.*;

import java.awt.Color;
import java.awt.FontMetrics;

/**
 * This object holds a set of colors that work well together. It is
 * intended to provide programmers with an abstraction so that the
 * job of assigning colors to interface elements can be made without
 * modifying interface code.<p>
 *
 * The following named colors are provided in a color scheme:
 * <dl>
 * <dt> base
 * <dd> The base color is the color that is used as the "dominant" color 
 *      for graphical objects. For example, a button's text is drawn on top 
 *      of this color when the button is "up".  
 * <dt> highlight
 * <dd> A lighter rendition of the base color used to create a highlight in 
 *      pseudo 3D effects.
 * <dt> shadow
 * <dd> A darker rendition of the base color used to create a shadow in pseudo
 *      3D effects.
 * <dt> background
 * <dd> The color used for background (or inset) <I>items</I> in a drawing 
 *      scheme (rather than as a typical background area per se).  For example,
 *      the background of a slider (the "groove" that the thumb slides in)
 *      is drawn in this color.  [This name probably needs to be changed.]
 * <dt>
 * <dd> Note: the colors: base, highlight, shadow, and background are designed 
 *      to be related, typically appearing to be the same material but with 
 *      different lighting.
 * <dt> foreground
 * <dd> The color normally drawn over the base color for foreground items such
 *      as textual labels.  This color needs to contrast with, but not clash 
 *      with the base color.
 * <dt> text_background
 * <dd> The color that serves as the background for text editing areas.
 * <dt> splash
 * <dd> A color which is designed to contrast with, and be significantly
 *      different from, the base, highlight, shadow, and background color 
 *      scheme.  This is used for indicators such as found inside a selected
 *      radio button or check box.
 * </dl>
 *
 * @author Ian Smith
 */
public class color_scheme {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** text_background color for the color scheme */
  protected Color _text_background;

  /** foreground color for the color scheme */
  protected Color _foreground;

  /** background color for the color scheme */
  protected Color _background;

  /** base color for the color scheme */
  protected Color _base;

  /** shadow color for the color scheme */
  protected Color _shadow;

  /** highlight color for the color scheme */
  protected Color _highlight;

  /** splash color for the color scheme */
  protected Color _splash;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  //XXX We still need a constructor which derives the highlight and shadow 
  //    colors from the base. XXX

  /**
   * Construct a new color scheme by providing all the colors.
   * 
   * @param Color text_bg    the background for text editing and displaying 
   *                         large amounts of text for reading.
   * @param Color fg         the foreground color for this scheme.
   * @param Color bg         a color with high contrast for use with the 
   *                         foreground color but one which will not be used for
   *                         display large amounts of text
   * @param Color base_color the basic color for this interface. It should be 
   *                         well matched with the shadow and highlight color 
   *                         to allow beveling.
   * @param Color shad       the color used for putting shadows on the object 
   *                         drawn with the base color.
   * @param Color high       the color used for putting highlights on the 
   *                         object drawn with the base color
   * @param Color splash     the color used for special effects (such as the 
   *                         center of a radio button, or the checkmark on a 
   *                         checkbox) in this scheme.
   */
  public color_scheme(Color text_bg, Color fg, Color bg, Color base_color, 
		      Color shad, Color high, Color splash) {
    _text_background = text_bg;
    _foreground= fg;
    _background = bg;
    _base = base_color;
    _shadow = shad;
    _highlight = high;
    _splash = splash;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Retrieve the text background color.
   * @return Color the text background color
   */
  public Color text_background() { return _text_background;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Retrieve the background color.
   * @return Color the background color
   */
  public Color background() { return _background;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Retrieve the foreground color.
   * @return Color the foreground color
   */
  public Color foreground() { return _foreground;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Retrieve the base color.
   * @return Color the base color
   */
  public Color base() { return _base;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Retrieve the highlight color.
   * @return Color the highlight color
   */
  public Color highlight() { return _highlight;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Retrieve the shadow color.
   * @return Color the shadow color
   */
  public Color shadow() { return _shadow;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Retrieve the splash color.
   * @return Color the splash color
   */
  public Color splash() { return _splash;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

}

/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
